from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from models.enums import  setLongTextToElideFormat,PkgStates,PKG_NAME
from models.apkinfo import *
from models.application import *
from utils import commontools
from ui.circular_progress_bar import *
from ui.uninstall_btn import *
from ui.shot_detail import *
import gettext
gettext.bindtextdomain("kylin-software-center", "/usr/share/locale")
gettext.textdomain("kylin-software-center")
_ = gettext.gettext

class Download_Ranking_card(QWidget):
    send_download_progress = pyqtSignal(int)
    puase_download = pyqtSignal(Application)
    puase_apk_download = pyqtSignal(ApkInfo)
    check_all_check_statue = pyqtSignal()

    def __init__(self, appmsg,num,ca,mw,parent=None):
        QWidget.__init__(self, parent)
        self.appmsg = appmsg
        self.app_dict = None
        self.pre_progress = 0
        self.num = num
        self.ca = ca
        self.mw = mw
        self.font = QFont()
        self.font.setPixelSize(14)
        self.m_font = QFontMetrics(self.font)
        self.narrow_btn_time = QTimer()
        self.init_ui()
        self.narrow_btn_time.timeout.connect(self.move_btn)
        self.mw.sigenal_show_downloading_widget.connect(self.show_downloading_widget)


    def init_ui(self):
        self.resize(237, 65)
        self.setWindowFlags(Qt.FramelessWindowHint)
        self.setAttribute(Qt.WA_TranslucentBackground)
        self.setStyleSheet("QFrame{border:0px solid #ff0000;border-radius: 12px;background-color:transparent;}")
        # self.setAutoFillBackground(True)
        # palette = QPalette()
        # palette.setColor(self.backgroundRole(), QColor(0, 255, 255))
        # self.setPalette(palette)

        self.icon_label = QLabel(self)
        self.icon_label.setFixedSize(48,48)
        self.icon_label.move(0,13)
        iconpath = commontools.get_big_icon_path(self.appmsg.pkgname)
        photo = Photo(iconpath, self.icon_label.width(),
                      self.icon_label.height(),
                      self.icon_label)
        photo.shot_photo_change(iconpath)
        # photo.resize_image()
        photo.show()
        self.icon_label.setStyleSheet("QLabel{border-radius: 8px;background-color:transparent;}")
        # self.icon_label.setStyleSheet("QLabel{border-radius: 8px;border-image: url('"+iconpath+"') 0 0 0 0 stretch strectch;background-color:transparent;}")

        self.num_label = QLabel(self)
        self.num_label.setAttribute(Qt.WA_TranslucentBackground)
        self.num_label.setFixedSize(10,14)
        self.num_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 500;color: #303133;}")
        self.num_label.setText(str(self.num))
        self.num_label.move(self.icon_label.x()+self.icon_label.width()+6,self.icon_label.y()+6)

        self.appname_label = QLabel(self)
        self.appname_label.setAttribute(Qt.WA_TranslucentBackground)
        self.appname_label.setAlignment(Qt.AlignVCenter)
        self.appname_label.resize(110,19)
        self.appname_label.setMaximumWidth(110)
        if(Globals.LAN == "zh_CN"):
            if self.appmsg.displayname_cn != '' and self.appmsg.displayname_cn is not None and self.appmsg.displayname_cn != 'None':
                setLongTextToElideFormat(self.appname_label, self.appmsg.displayname_cn)
                if ("…" in self.appname_label.text()):
                    self.appname_label.setToolTip(self.appmsg.displayname_cn)
            else:
                setLongTextToElideFormat(self.appname_label, self.appmsg.pkgname)
                if ("…" in self.appname_label.text()):
                    self.appname_label.setToolTip(self.appmsg.pkgname)
        else:
            setLongTextToElideFormat(self.appname_label, self.appmsg.pkgname)
            if ("…" in self.appname_label.text()):
                self.appname_label.setToolTip(self.appmsg.pkgname)
        # self.appname_label.adjustSize()
        self.appname_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 500;color: #303133;text-align: left;}")
        self.appname_label.move(self.num_label.x()+self.num_label.width()+6,self.num_label.y()-3)

        self.app_categories = QLabel(self)
        self.app_categories.setAttribute(Qt.WA_TranslucentBackground)
        self.app_categories.setFixedSize(65,15)
        self.app_categories.setAlignment(Qt.AlignVCenter)
        self.app_categories.setStyleSheet("QLabel{font-size: 12px;font-weight: 400;color: #8F9399;}")
        if(self.appmsg.category_name_cn !="" and self.appmsg.category_name_cn !="None" and self.appmsg.category_name_cn !=None  and Globals.LAN == "zh_CN"):
            self.app_categories.setText(self.appmsg.category_name_cn.split("、")[0])
        else:
            self.app_categories.setText(self.appmsg.category_name.split("、")[0])
        # self.app_categories.adjustSize()
        self.app_categories.move(self.appname_label.x(),self.appname_label.y()+self.appname_label.height()+8)

        self.btn = QPushButton(self)
        self.btn.setFocusPolicy(Qt.NoFocus)
        self.btn.setStyleSheet("QPushButton{font-size: 12px;font-weight: 400;color: #FFFFFF;line-height: 12px;border-radius: 11px;border: 0px solid #3790FA;background: #3790FA;}QPushButton:hover{background: #40A9FB;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}QPushButton:pressed{background: #296CD9;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}")
        self.btn.resize(48, 22)
        self.btn.move(int(self.width()-self.btn.width()),int((self.icon_label.height()-self.btn.height())/2) + self.icon_label.y())

        self.line_label = QLabel(self)
        self.line_label.resize(self.width(),1)
        # self.line_label.setStyleSheet("QLabel{border: 1px solid rgba(151,151,151,0.2);}")
        self.line_label.move(0,self.height()-2)
        self.line_label.show()

        self.down_progress = Circular_Progress_Bar(22,22,self.appmsg.pkgname,self)
        self.down_progress.update_progress(self.appmsg.percent)
        self.send_download_progress.connect(self.down_progress.GenUpdate)
        self.down_progress.btn.clicked.connect(self.slot_puase_download)
        self.down_progress.cont_btn.clicked.connect(lambda :self.mw.re_download_app(self.appmsg))
        self.down_progress.move(int(self.width()-self.down_progress.width()), self.btn.y())
        self.down_progress.hide()
        self.remove_btn = uninstall_btn(self)
        self.remove_btn.move(self.btn.x() + int((self.btn.width()-self.remove_btn.width())/2), self.btn.y())
        self.remove_btn.hide()
        self.slot_set_dark_mode()
        self.check_btn_text()

    #
    # 函数：设置深色模式
    #
    def slot_set_dark_mode(self):
        if (Globals.THEME_COLOR == 0):
            # self.setStyleSheet("QFrame{border:0px solid #ff0000;border-radius: 12px;background-color:#ffffff;}")
            self.num_label.setStyleSheet(
                "QLabel{font-size: 14px;font-weight: 500;color: #303133;line-height: 20px;}")
            self.appname_label.setStyleSheet(
                "QLabel{font-size: 14px;font-weight: 400;color: #303133;line-height: 17px;}")
            self.app_categories.setStyleSheet(
                "QLabel{font-size: 12px;font-weight: 400;color: #8F9399;line-height: 17px;}")
            self.line_label.setStyleSheet("QLabel{border: 1px solid rgba(151,151,151,0.2);}")
        else:
            # self.setStyleSheet("QFrame{background: rgba(46, 46, 46, 0.88);border-radius: 12px;}")
            self.num_label.setStyleSheet(
                "QLabel{font-size: 14px;font-weight: 500;color: #F9F9F9;line-height: 20px;}")
            self.appname_label.setStyleSheet(
                "QLabel{font-size: 14px;font-weight: 400;color: #F9F9F9;line-height: 17px;}")
            self.app_categories.setStyleSheet(
                "QLabel{font-size: 12px;font-weight: 400;color: #8F9399;line-height: 17px;}")
            self.line_label.setStyleSheet("QLabel{border: 1px solid #393939;}")
        self.down_progress.reflash_color()
        # QApplication.processEvents()
        # self.get_btn_color()

    #
    # 函数：判断软件的desktop文件
    #
    def check_app_desktop(self, pkgname):
        # 对一些特殊软件单独处理

        fullcmd = ""

        desktopfile = "/usr/share/applications/" + pkgname + ".desktop"
        desktopfile1 = "/usr/share/applications/kde4/" + pkgname + ".desktop"

        if os.path.exists(desktopfile):
            return True
        elif os.path.exists(desktopfile1):
            return True
        else:
            if pkgname in Globals.PKG_NAME:
                pkgname = Globals.PKG_NAME[pkgname]

                desktopfile = "/usr/share/applications/" + pkgname + ".desktop"

                if os.path.exists(desktopfile):
                    return True
                else:
                    desktopfile = "/usr/share/applications/kde4/" + pkgname + ".desktop"
                    if (os.path.exists(desktopfile)):
                        return True
                    else:
                        return False
            else:
                return False

    def change_btn_width(self):
        s = self.m_font.boundingRect(self.btn.text()).size().width()
        if (s +18< 48):
            self.btn.setFixedSize(48,22)
        else:
            self.btn.setFixedSize(s+18,22)
        # print("00000",self.btn.width())
        self.btn.move(int(self.width()-self.btn.width()),int((self.icon_label.height()-self.btn.height())/2) + self.icon_label.y())

    def check_btn_text(self):
        self.btn.resize(48, 22)
        self.btn.move(int(self.width() - self.btn.width()),
                      int((self.icon_label.height() - self.btn.height()) / 2) + self.icon_label.y())
        self.remove_btn.stop_loading()
        self.remove_btn.hide()
        self.down_progress.hide()
        self.btn.setCheckable(True)
        self.btn.setEnabled(True)
        if (self.appmsg.status == PkgStates.REMOVING):
            self.slot_uninstall(self.appmsg.pkgname, "uninstalling")
            return
        if (type(self.appmsg) != ApkInfo):
            try:
                self.package = self.ca[self.appmsg.pkgname]
            except:
                self.package = None
            if (self.package != None):
                if (self.package.is_installed == True and self.package.is_upgradable == True):
                    self.btn.setText(_("update"))
                    self.change_btn_width()
                    self.btn.setStyleSheet("QPushButton{font-size: 12px;font-weight: 400;color: #3790FA;line-height: 12px;border-radius: 11px;background: rgba(55, 144, 250, 0.12);}QPushButton:hover{background: #3790FA;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}QPushButton:pressed{background: #296CD9;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}")
                elif (self.package.is_installed == False):
                    self.btn.setText(_("down"))
                    self.change_btn_width()
                    self.btn.setStyleSheet("QPushButton{font-size: 12px;font-weight: 400;color: #FFFFFF;line-height: 12px;border-radius: 11px;border: 0px solid #3790FA;background: #3790FA;}QPushButton:hover{background: #40A9FB;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}QPushButton:pressed{background: #296CD9;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}")
                elif (self.package.is_installed == True and self.package.is_upgradable == False):
                    if (self.appmsg.pkgname == "kylin-installer"):
                        has_desktop = False
                    else:
                        has_desktop = self.check_app_desktop(self.appmsg.pkgname)
                    if (has_desktop == True):
                        self.btn.setText(_("Open"))
                        self.change_btn_width()
                        self.btn.setStyleSheet("QPushButton{font-size: 12px;font-weight: 400;color: #3790FA;line-height: 12px;border-radius: 11px;background: rgba(55, 144, 250, 0.12);}QPushButton:hover{background: #3790FA;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}QPushButton:pressed{background: #296CD9;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}")
                    else:
                        self.btn.setText(_("installed"))
                        self.change_btn_width()
                        self.btn.setEnabled(False)
                        self.btn.setStyleSheet("QPushButton{font-size: 12px;font-weight: 400;color: #3790FA;line-height: 12px;background: rgba(55, 144, 250, 0.12);border-radius: 11px;}")
                    pass
            else:
                self.btn.setText(_("down"))
                self.change_btn_width()
                self.btn.setStyleSheet("QPushButton{font-size: 12px;font-weight: 400;color: #FFFFFF;line-height: 12px;border-radius: 11px;border: 0px solid #3790FA;background: #3790FA;}QPushButton:hover{background: #40A9FB;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}QPushButton:pressed{background: #296CD9;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}")
        else:
            if (self.appmsg != None):
                if (self.appmsg.is_installed == True and self.appmsg.is_upgradable == True):
                    self.btn.setText(_("update"))
                    self.change_btn_width()
                    self.btn.setStyleSheet("QPushButton{font-size: 12px;font-weight: 400;color: #3790FA;line-height: 12px;border-radius: 11px;background: rgba(55, 144, 250, 0.12);}QPushButton:hover{background: #3790FA;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}QPushButton:pressed{background: #296CD9;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}")
                elif (self.appmsg.is_installed == False):
                    self.btn.setText(_("down"))
                    self.change_btn_width()
                    self.btn.setStyleSheet("QPushButton{font-size: 12px;font-weight: 400;color: #FFFFFF;line-height: 12px;border-radius: 11px;border: 0px solid #3790FA;background: #3790FA;}QPushButton:hover{background: #40A9FB;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}QPushButton:pressed{background: #296CD9;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}")
                elif (self.appmsg.is_installed == True and self.appmsg.is_upgradable == False):
                    self.btn.setText(_("Open"))
                    self.change_btn_width()
                    self.btn.setStyleSheet("QPushButton{font-size: 12px;font-weight: 400;color: #3790FA;line-height: 12px;border-radius: 11px;background: rgba(55, 144, 250, 0.12);}QPushButton:hover{background: #3790FA;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}QPushButton:pressed{background: #296CD9;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}")
                    pass
            else:
                self.btn.setText(_("installed"))
                self.change_btn_width()
        self.btn.show()
        # self.get_btn_color()
    #
    #函数：获取btn的颜色
    #
    def get_btn_color(self):
        if(self.btn.text() == _("down")):
            if (Globals.THEME_COLOR == 0):
                self.btn.setStyleSheet("QPushButton{font-size: 12px;font-weight: 400;color: #FFFFFF;line-height: 12px;border-radius: 11px;border: 0px solid #3790FA;background: #3790FA;}QPushButton:hover{background: #40A9FB;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}QPushButton:pressed{background: #296CD9;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}")
            else:
                self.btn.setStyleSheet("QPushButton{font-size: 12px;font-weight: 400;color: #3790FA;line-height: 12px;border-radius: 11px;border: 1px solid #3790FA;}QPushButton:hover{background: #3790FA;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}QPushButton:pressed{background: #296CD9;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}")
        elif(self.btn.text() == _("cont")):
            self.btn.setStyleSheet("QPushButton{font-size: 12px;font-weight: 400;color: #3790FA;line-height: 12px;border-radius: 11px;background: rgba(55, 144, 250, 0.12);}QPushButton:hover{background: #3790FA;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}QPushButton:pressed{background: #3790FA;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}")
        elif(self.btn.text() == _("installing") or self.btn.text() == _("installed")):
            self.btn.setStyleSheet("QPushButton{font-size: 12px;font-weight: 400;color: #3790FA;line-height: 12px;background: rgba(55, 144, 250, 0.12);border-radius: 11px;}")
        else:
            if (Globals.THEME_COLOR == 0):
                self.btn.setStyleSheet("QPushButton{font-size: 12px;font-weight: 400;color: #3790FA;line-height: 12px;border-radius: 11px;background: rgba(55, 144, 250, 0.12);}QPushButton:hover{background: #3790FA;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}QPushButton:pressed{background: #296CD9;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}")
            else:
                self.btn.setStyleSheet("QPushButton{font-size: 12px;font-weight: 400;color: #3790FA;line-height: 12px;border-radius: 11px;border: 1px solid #3790FA;}QPushButton:hover{background: #3790FA;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}QPushButton:pressed{background: #296CD9;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}")

    def reload_btn(self,appname):
        if(appname == self.appmsg.pkgname):
            self.check_btn_text()

    def show_downloading_widget(self,appname,is_cancel = False,is_puase = False):
        if(appname == self.appmsg.pkgname):
            if( self.appmsg.status != PkgStates.NOTHING):
                if(is_cancel == False):
                    if (is_puase == False):
                        if (self.appmsg.status != PkgStates.INSTALLING):
                            self.btn.hide()
                            self.down_progress.show()
                            self.down_progress.btn.show()
                            self.down_progress.cont_btn.hide()
                    else:
                        self.btn.setText(_("cont"))
                        self.change_btn_width()
                        self.btn.setStyleSheet("QPushButton{font-size: 12px;font-weight: 400;color: #3790FA;line-height: 12px;border-radius: 11px;background: rgba(55, 144, 250, 0.12);}QPushButton:hover{background: #3790FA;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}QPushButton:pressed{background: #3790FA;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}")
                        self.btn.show()
                        self.down_progress.show()
                else:
                    self.btn.resize(52, 24)
                    self.btn.move(self.width() - self.btn.width() - 20, self.height() - self.btn.height() - 20)
                    self.btn.show()
                    self.down_progress.hide()
            else:
                self.check_btn_text()
        # self.get_btn_color()

    def slot_get_download_progress(self,appname,progress):
        if(appname == self.appmsg.pkgname):
            if(progress == 100):
                self.btn.setEnabled(False)
                self.narrow_btn_time.stop()
                self.btn.resize(48, 22)
                self.btn.move(int(self.width() - self.btn.width()),
                              int((self.icon_label.height() - self.btn.height()) / 2) + self.icon_label.y())
                self.down_progress.hide()
                self.btn.setCheckable(False)
                self.btn.setText(_("installing"))
                self.change_btn_width()
                self.btn.setStyleSheet("QPushButton{font-size: 12px;font-weight: 400;color: #3790FA;line-height: 12px;border-radius: 11px;background: rgba(55, 144, 250, 0.12);}QPushButton:hover{background: #3790FA;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}QPushButton:pressed{background: #296CD9;border-radius: 11px;color: #ffffff;font-weight: 400;border:0px;}")
                self.btn.show()
            else:
                if (self.appmsg.status != PkgStates.PUASE):
                    if (progress - self.pre_progress >= 2):
                        self.send_download_progress.emit(progress)
                        self.pre_progress = progress
                    else:
                        self.send_download_progress.emit(self.pre_progress)

    # 处理：卸载界面的显示
    def slot_uninstall(self, appname, state):
        if (appname == self.appmsg.pkgname):
            self.btn.resize(48, 22)
            self.btn.move(int(self.width() - self.btn.width()),
                          int((self.icon_label.height() - self.btn.height()) / 2) + self.icon_label.y())
            if (state == "uninstalling"):
                self.btn.setText(_("uninstalling"))
                self.change_btn_width()
                self.btn.hide()
                self.remove_btn.start_loading()
                self.remove_btn.show()
            else:
                self.btn.setText(_("uninstalled"))
                self.change_btn_width()
                self.btn.setStyleSheet(
                    "QPushButton{font-size: 12px;font-weight: 400;color: #F8F8F8;line-height: 12px;background: #C0C4CC;border-radius: 11px;}")

    def change_btn_state(self,appname,state):
        if(appname == self.appmsg.pkgname):
            if(state == PkgStates.NOTHING or state == PkgStates.REMOVED):
                self.appmsg.status = PkgStates.NOTHING
                self.check_btn_text()
                self.check_all_check_statue.emit()
            if(state == PkgStates.INSTALLING):
                self.slot_get_download_progress(appname,100)
            if(state == PkgStates.REMOVING):
                self.slot_uninstall(appname,"uninstalling")
            if(state == PkgStates.INSTALLED):
                self.appmsg.is_installed = True
                self.appmsg.is_upgradable = False
                self.appmsg.status = PkgStates.NOTHING
                self.reload_btn(appname)
                self.check_all_check_statue.emit()
            if(state == PkgStates.DOWNLOADING):
                if(self.btn.isHidden() == False):
                    self.down_progress.init_down = True
                    self.down_progress.persent = 0
                    self.pre_progress = 0
                if (self.appmsg.status != PkgStates.NOTHING):
                    if (self.btn.text() != ""):
                        self.narrow_btn_time.start(1)
                    else:
                        self.btn.hide()
                        self.down_progress.btn.show()
                        self.down_progress.cont_btn.hide()
                        self.down_progress.show()
                # self.show_downloading_widget(appname)
            if (state == PkgStates.PUASE):
                # self.show_downloading_widget(appname,False,True)
                self.appmsg.status = PkgStates.PUASE
                self.btn.hide()
                self.down_progress.show()
                self.down_progress.puase_download(self.appmsg.percent)
                self.check_all_check_statue.emit()

    def move_btn(self):
        self.narrow_btn_time.stop()
        self.btn.setText("")
        w = self.btn.width()
        h = self.btn.height()
        x = self.btn.x()
        y = self.btn.y()
        if(self.appmsg.status != PkgStates.NOTHING):
            if(self.btn.width()>self.btn.height()):
                self.btn.hide()
                self.btn.setFixedSize(w-1,h)
                self.btn.move(x+1,y)
                self.btn.show()
                self.narrow_btn_time.start(4)
            else:
                self.show_downloading_widget(self.appmsg.pkgname)


    #
    # 函数：处理点击暂停
    #
    def slot_puase_download(self):
        if (type(self.appmsg) != ApkInfo):
            self.puase_download.emit(self.appmsg)
        else:
            self.puase_apk_download.emit(self.appmsg)


#!/usr/bin/python3
# -*- coding: utf-8 -*-


from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
import math
from models.globals import Globals
import gettext
gettext.bindtextdomain("kylin-software-center", "/usr/share/locale")
gettext.textdomain("kylin-software-center")
_ = gettext.gettext

class Search_records(QFrame):
    clicked_item = pyqtSignal(QListWidgetItem)
    space = 8
    def __init__(self,w,parent = None):
        QFrame.__init__(self,parent)
        self.w = w
        self.init_ui()
        self.mw = parent
        self.enter_item = None
        self.clicked_item.connect(self.mw.slot_click_search_item)
        self.paintEvent = self.set_paintevent

    def init_ui(self):
        self.resize(self.w+20,0)
        self.setWindowFlags(Qt.FramelessWindowHint)
        self.setAttribute(Qt.WA_TranslucentBackground)
        # self.setStyleSheet("QFrame{width: 232px;height: 106px;background: #FFFFFF;border-radius: 4px;}")
        self.widget = QWidget(self)
        # self.widget.setStyleSheet("QWidget{background: #FFFFFF;border-radius: 4px;}")
        self.widget.resize(self.w,0)
        self.widget.move(10,10)
        #self.setAttribute(Qt.WA_TranslucentBackground)

        self.item_list_widget = QListWidget(self.widget)
        self.item_list_widget.resize(self.w-4,0)
        self.item_list_widget.move(2,2)
        # self.item_list_widget.setStyleSheet("QListWidget{background-color:#ffffff;border:0px solid #cfcfcf;border-radius: 4px;}QListWidget::item{height: 30px;border-radius: 2px;background:#FFFFFF;}QListWidget::item:hover{background:#296CD9;}")
        self.slot_show_dark_mode()

    def slot_show_dark_mode(self):
        if(Globals.THEME_COLOR == 0):
            if(Globals.RIGHT_ANGLE):
                self.setStyleSheet("QFrame{background: transparent;}")
                self.widget.setStyleSheet("QWidget{background: #FFFFFF;}")
            else:
                self.setStyleSheet("QFrame{background: transparent;border-radius: 8px;}")
                self.widget.setStyleSheet("QWidget{background: #FFFFFF;border-radius: 8px;}")
            self.item_list_widget.setStyleSheet(
                "QListWidget{background-color:#ffffff;border:0px solid #cfcfcf;border-radius: 8px;}QListWidget::item{height: 36px;background:#FFFFFF;border-radius: 6px;}QListWidget::item:hover{background:#3790FA;border-radius: 6px;}")
        else:
            if (Globals.RIGHT_ANGLE):
                self.setStyleSheet("QFrame{background: #3D3D41;}")
                self.widget.setStyleSheet("QWidget{background: #3D3D41;}")
            else:
                self.setStyleSheet("QFrame{background: #3D3D41;border-radius: 8px;}")
                self.widget.setStyleSheet("QWidget{background: #3D3D41;border-radius: 8px;}")
            self.item_list_widget.setStyleSheet("QListWidget{background-color:#3D3D41;border:0px solid #cfcfcf;border-radius: 8px;}QListWidget::item{height: 36px;background:#3D3D41;border-radius: 6px;}QListWidget::item:hover{background:#296CD9;border-radius: 6px;}")
        for i in range(self.item_list_widget.count()):
            item = self.item_list_widget.item(i)
            get_widget = self.item_list_widget.itemWidget(item)
            get_widget.slot_show_dark_mode()

    def slot_change_list_style(self,enter_widget):
        for i in range(self.item_list_widget.count()):
            item = self.item_list_widget.item(i)
            get_widget = self.item_list_widget.itemWidget(item)
            if(get_widget != None):
                if(get_widget == enter_widget):
                    get_widget.appname_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #ffffff;border-radius:6px;}")
                    get_widget.setStyleSheet("QFrame{border-radius:6px;background:#296CD9;}")
                else:
                    if (Globals.THEME_COLOR == 0):
                        get_widget.setStyleSheet("QFrame{border-radius:6px;background:#FFFFFF;}")
                        get_widget.appname_label.setStyleSheet(
                            "QLabel{font-size: 14px;font-weight: 400;color: #303133;border-radius:6px;}")
                    else:
                        get_widget.setStyleSheet("QFrame{border-radius:6px;background:#3D3D41;}")
                        get_widget.appname_label.setStyleSheet(
                            "QLabel{font-size: 14px;font-weight: 400;color: #F9F9F9;border-radius:6px;}")

    def key_up(self):
        if (self.enter_item == None):
            self.enter_item = self.item_list_widget.count() - 1
            item = self.item_list_widget.item(self.enter_item)
            get_item_widget = self.item_list_widget.itemWidget(item)
            get_item_widget.setStyleSheet("QFrame{border-radius:6px;background:#3790FA;}")
            get_item_widget.appname_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #ffffff;}")
        else:
            if (self.enter_item - 1 >= 0):
                pre_item_num = self.enter_item
                self.enter_item = self.enter_item - 1
            else:
                pre_item_num = self.enter_item
                self.enter_item = self.item_list_widget.count() - 1
            pre_item = self.item_list_widget.item(pre_item_num)
            get_pre_item_widget = self.item_list_widget.itemWidget(pre_item)
            if (Globals.THEME_COLOR == 0):
                get_pre_item_widget.setStyleSheet("QFrame{border-radius:2px;background:#FFFFFF;}")
                get_pre_item_widget.appname_label.setStyleSheet(
                    "QLabel{font-size: 14px;font-weight: 400;color: #303133;}")
            else:
                get_pre_item_widget.setStyleSheet("QFrame{border-radius:6px;background:#3D3D41;}")
                get_pre_item_widget.appname_label.setStyleSheet(
                    "QLabel{font-size: 14px;font-weight: 400;color: #F9F9F9;}")
            item = self.item_list_widget.item(self.enter_item)
            get_item_widget = self.item_list_widget.itemWidget(item)
            get_item_widget.appname_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #ffffff;}")
            get_item_widget.setStyleSheet("QFrame{border-radius:6px;background:#3790FA;}")


    def key_down(self):
        if (self.enter_item == None):
            self.enter_item = 0
            item = self.item_list_widget.item(self.enter_item)
            get_item_widget = self.item_list_widget.itemWidget(item)
            get_item_widget.setStyleSheet("QFrame{border-radius:6px;background-color:#3790FA;}")
            get_item_widget.appname_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #ffffff;}")

        else:
            if (self.enter_item + 1 <= self.item_list_widget.count() - 1):
                pre_item_num = self.enter_item
                self.enter_item = self.enter_item + 1
            else:
                pre_item_num = self.enter_item
                self.enter_item = 0
            pre_item = self.item_list_widget.item(pre_item_num)
            get_pre_item_widget = self.item_list_widget.itemWidget(pre_item)
            if (Globals.THEME_COLOR == 0):
                get_pre_item_widget.setStyleSheet("QFrame{border-radius:6px;background:#FFFFFF;}")
                get_pre_item_widget.appname_label.setStyleSheet(
                    "QLabel{font-size: 14px;font-weight: 400;color: #303133;}")
            else:
                get_pre_item_widget.setStyleSheet("QFrame{border-radius:6px;background-color:#3D3D41;}")
                get_pre_item_widget.appname_label.setStyleSheet(
                    "QLabel{font-size: 14px;font-weight: 400;color: #F9F9F9;}")
            item = self.item_list_widget.item(self.enter_item)
            get_item_widget = self.item_list_widget.itemWidget(item)
            get_item_widget.appname_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #ffffff;}")
            get_item_widget.setStyleSheet("QFrame{border-radius:6px;background:#3790FA;}")

    def keyPressEvent(self, keyevent):
        key = keyevent.key()
        if(key == Qt.Key_Up):
            self.key_up()
        elif(key == Qt.Key_Down):
            self.key_down()
        elif(key == Qt.Key_Enter or key == Qt.Key_Return):
            if(self.enter_item != None):
                self.clicked_item.emit(self.item_list_widget.item(self.enter_item))
        elif(key >= Qt.Key_A and key <= Qt.Key_Z):
            self.mw.title_bar.search_line.setFocus()
            self.mw.title_bar.search_line.setText(self.mw.title_bar.search_line.text()+keyevent.text())
        elif(key == Qt.Key_Backspace):
            self.mw.title_bar.search_line.setFocus()
            self.mw.title_bar.search_line.backspace()


    def set_paintevent(self,event):
        painter = QPainter(self)
        m_defaultBackgroundColor = QColor(qRgb(255, 255, 255))
        m_defaultBackgroundColor.setAlpha(50)
        path = QPainterPath()
        path.setFillRule(Qt.WindingFill)
        path.addRoundedRect(10, 10, self.width() - 20, self.height() - 20, 8, 8)

        painter.setRenderHint(QPainter.Antialiasing, True)
        painter.fillPath(path, QBrush(QColor(m_defaultBackgroundColor.red(),
                                             m_defaultBackgroundColor.green(),
                                             m_defaultBackgroundColor.blue())))

        color = QColor(0, 0, 0, 20)
        i = 0
        while i < 4:
            path = QPainterPath()
            path.setFillRule(Qt.WindingFill)
            path.addRoundedRect(10 - i, 10 - i, self.width() - (10 - i) * 2, self.height() - (10 - i) * 2, 8, 8)
            color.setAlpha(int(100 - math.sqrt(i) * 50))
            painter.setPen(color)
            painter.drawPath(path)
            i = i + 1

        painter.setRenderHint(QPainter.Antialiasing)


class line_widget(QFrame):
    enter_widget = pyqtSignal(QFrame)
    def __init__(self,w,parent = None):
        QFrame.__init__(self,parent)
        self.w = w
        self.init_ui()

    def init_ui(self):
        self.resize(self.w,36)
        self.setWindowFlags(Qt.FramelessWindowHint)
        self.setStyleSheet("QFrame{background-color:transparent;border-radius:6px;}")

        self.appname_label = QLabel(self)
        self.appname_label.move(20,0)
        self.appname_label.setFixedSize(self.width()-20,self.height())
        self.appname_label.setAttribute(Qt.WA_TranslucentBackground)
        self.appname_label.setAlignment(Qt.AlignVCenter)
        self.appname_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #303133;border-radius:6px;}")
        self.slot_show_dark_mode()

    def setLongTextToElideFormat(self,label, text):
        if text[len(text) - 1] == '\n':
            text = text.rstrip()
        metrics = QFontMetrics(label.font())
        elidedText = metrics.elidedText(text, Qt.ElideRight, label.maximumWidth())
        if "…" in elidedText:
            label.setToolTip(text)
        self.appname = text
        label.setText(elidedText)

    def slot_show_dark_mode(self):
        if (Globals.THEME_COLOR == 0):
            self.appname_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #303133;border-radius:6px;}")
            self.setStyleSheet("QFrame{background-color:#FFFFFF;border-radius:6px;}")
        else:
            self.appname_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #F9F9F9;border-radius:6px;}")
            self.setStyleSheet("QFrame{background-color:transparent;border-radius:6px;}")

    def enterEvent(self, event):
        self.enter_widget.emit(self)
        self.appname_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #ffffff;border-radius:6px;}")
        self.setStyleSheet("QFrame{border-radius:6px;background-color:#3790FA;}")

    def leaveEvent(self,event):
        if (Globals.THEME_COLOR == 0):
            self.appname_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #303133;border-radius:6px;}")
            self.setStyleSheet("QFrame{background-color:#FFFFFF;border-radius:6px;}")
        else:
            self.appname_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #F9F9F9;border-radius:6px;}")
            self.setStyleSheet("QFrame{background-color:transparent;border-radius:6px;}")



#!/usr/bin/python3
# -*- coding: utf-8 -*-


from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from models.enums import BUTTON_ENTER,BUTTON_LEAVE,BUTTON_PRESSED,BUTTON_DISABLE
from ui.popular_app_card import *
from ui.systembutton import *
from ui.ad_widget import *
from models.application import *
from models.apkinfo import *
from ui.select_widget import *
from models.enums import setLongTextToElideFormat
import os
import sip
import gettext
gettext.bindtextdomain("kylin-software-center", "/usr/share/locale")
gettext.textdomain("kylin-software-center")
_ = gettext.gettext

class Category_Widget(QFrame):
    clicked_btn = True
    is_first_click = True
    click_type_btn = pyqtSignal(str,bool)
    click_category_ad = pyqtSignal(Application,bool)
    click_category_ad_apk = pyqtSignal(ApkInfo,bool)
    click_none_card = pyqtSignal()
    card_width = 248
    card_height = 124
    card_space = 15
    widget_space = 50
    def __init__(self,w,h,parent = None):
        click_ad_card = pyqtSignal(Application)
        QFrame.__init__(self,parent)
        self.w = w
        self.h = h
        self.m_parent = parent
        self.init_ui()
        self.init_style()

    def init_ui(self):
        self.resize(self.w, self.h)
        self.setWindowFlags(Qt.FramelessWindowHint)
        #self.setAttribute(Qt.WA_TranslucentBackground)

        # self.btn_widget = CategoryBar(Globals.ALL_CATEGORIES,self,False)
        tmp = [val for val in Globals.ALL_CATEGORIES if val in Globals.ORDINARY_CATEGORY]
        # self.btn_widget = CategoryBar(Globals.ORDINARY_CATEGORY,self,False)
        self.btn_widget = CategoryBar(tmp,self,False)
        self.btn_widget.setAttribute(Qt.WA_TranslucentBackground)
        self.btn_widget.resize((self.btn_widget.visiblecategorycount - 1) * ( self.btn_widget.itemwidth + self.btn_widget.categoryspacing) + self.btn_widget.itemwidth, 64)
        self.btn_widget.move(int((self.w - self.btn_widget.width()) / 2), 20)
        self.btn_widget.show()

        self.type_widget = QWidget(self)
        # self.type_widget.setAttribute(Qt.WA_TranslucentBackground)
        self.type_widget.resize(self.w,self.h-self.btn_widget.height()-16)
        self.type_widget.move(0,self.btn_widget.height()+self.btn_widget.y())
        self.type_widget.setStyleSheet("QWidget{background-color:transparent;border-radius:0px;}")
        self.type_widget.hide()

        w1 = int((self.type_widget.width()-self.widget_space-self.widget_space - 20) * (326 / (Globals.INIT_DETAIL_WIDTH -144-144 - 20)))
        w2 = int((self.type_widget.width()-self.widget_space-self.widget_space - 20) * (690 / (Globals.INIT_DETAIL_WIDTH -144-144 - 20)))
        self.category_ad_card = Advertisement_Widget(self.type_widget.width()-self.widget_space-self.widget_space,216,w1,w2,self.m_parent.mw,False,self.type_widget)
        self.category_ad_card.setAttribute(Qt.WA_TranslucentBackground)
        self.category_ad_card_height = 216
        self.category_ad_card.move(int((self.type_widget.width()-self.category_ad_card.width())/2),0)
        self.category_ad_card.show()
        self.category_ad_card.click_ad_card.connect(self.slot_click_ad_card)
        self.category_ad_card.click_ad_card_apk.connect(self.slot_click_ad_card)
        self.category_ad_card.click_none_app_ad_card.connect(self.slot_click_none_card)

        self.preview_ad_card = QLabel(self.type_widget)
        self.preview_ad_card.setAttribute(Qt.WA_TranslucentBackground)
        self.preview_ad_card.resize(self.type_widget.width()-self.widget_space*2,self.category_ad_card.height())
        self.preview_ad_card.move(int((self.type_widget.width() - self.preview_ad_card.width()) / 2), 0)
        photo = "./data/ads/preview_banner.png"
        pix2 = QPixmap(photo)
        fitpixmap_userIcon = pix2.scaled(self.preview_ad_card.width(), self.preview_ad_card.height(), Qt.IgnoreAspectRatio, Qt.SmoothTransformation)
        fitpixmap_userIcon = self.PixmapToRound(fitpixmap_userIcon,self.preview_ad_card.width(), self.preview_ad_card.height())
        self.preview_ad_card.setPixmap(fitpixmap_userIcon)
        self.preview_ad_card.hide()

        self.type_name_label = QLabel(self.type_widget)
        self.type_name_label.setAttribute(Qt.WA_TranslucentBackground)
        self.type_name_label.setFixedHeight(30)
        self.type_name_label.setStyleSheet("QLabel{font-size: 24px;font-weight: 500;color: #303133;line-height: 42px;}")
        self.type_name_label.setText(self.btn_widget.first_btn)
        self.type_name_label.adjustSize()
        self.type_name_label.move(self.widget_space,self.category_ad_card.height()+30)
        self.type_name_label.show()
        # self.type_name_label.hide()

        self.type_show_card_widget = QWidget(self.type_widget)
        self.type_show_card_widget.setAttribute(Qt.WA_TranslucentBackground)
        self.type_show_card_widget.resize(self.type_widget.width() - self.widget_space - self.widget_space,
                                          self.type_widget.height() - self.type_name_label.height())
        self.type_show_card_widget.move(self.widget_space, self.type_name_label.y() + self.type_name_label.height())
        self.type_show_card_widget.show()

        self.type_show_card_widget.children_count = 0

        self.choose_sorting_type_btn = QPushButton(self.type_widget)
        # self.choose_sorting_type_btn.setAttribute(Qt.WA_TranslucentBackground)
        self.choose_sorting_type_btn.setStyleSheet("QPushButton{border-radius: 13px;font-size: 14px;font-weight: 400;color: #3790FA;line-height: 20px;background: transparent;}QPushButton::hover{color: #3790FA;background: rgba(55, 144, 250, 0.12);font-weight: 400;}QPushButton:pressed{color: #3790FA;background: transparent;font-weight: 500;}")
        self.choose_sorting_type_btn.setFocusPolicy(Qt.NoFocus)
        self.type_sorting_btn = QPushButton(self.choose_sorting_type_btn)
        self.type_sorting_btn.setAttribute(Qt.WA_TranslucentBackground)
        self.type_sorting_btn.setFocusPolicy(Qt.NoFocus)
        self.type_sorting_btn.move(0,0)
        self.type_sorting_btn.setFixedSize(80, 26)
        self.type_sorting_btn.setStyleSheet("QPushButton{font-size: 14px;font-weight: 400;color: #3790FA;line-height: 20px;background: transparent;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: transparent;font-weight: 500;}")
        self.type_sorting_btn.setText(_("Defualt"))
        self.type_sorting_btn.adjustSize()
        self.type_sorting_btn.show()
        # self.type_sorting_btn.clicked.connect(lambda: self.slot_click_sorting_btn(1))

        self.pull_down_button = QPushButton(self.choose_sorting_type_btn)
        self.pull_down_button.setFocusPolicy(Qt.NoFocus)
        self.pull_down_button.setAttribute(Qt.WA_TranslucentBackground)
        self.pull_down_button.resize(16,16)
        self.pull_down_button.setStyleSheet("QPushButton{border:0px;background-image:url('res/show_search_black.png')}QPushButton:hover{border:0px;background-image:url('res/show_search_clicked.png')}QPushButton:pressed{border:0px;background-image:url('res/show_search_clicked.png')}")
        self.choose_sorting_type_btn.resize(self.type_sorting_btn.width()+self.pull_down_button.width()+5,26)
        self.pull_down_button.move(self.type_sorting_btn.x()+self.type_sorting_btn.width(),int((self.type_sorting_btn.height()-self.pull_down_button.height())/2))
        self.choose_sorting_type_btn.move(self.type_show_card_widget.width() - self.choose_sorting_type_btn.width()+self.type_show_card_widget.x()-10,self.type_name_label.y())
        self.choose_sorting_type_btn.clicked.connect(self.slot_choose_sorting_type)
        self.pull_down_button.clicked.connect(self.slot_choose_sorting_type)
        self.type_sorting_btn.clicked.connect(self.slot_choose_sorting_type)
        self.pull_down_button.show()

        self.rating_sorting_btn = QPushButton(self.type_widget)
        self.rating_sorting_btn.setFocusPolicy(Qt.NoFocus)
        self.rating_sorting_btn.setFixedSize(80, 26)
        self.rating_sorting_btn.setStyleSheet(
            "QPushButton{font-size: 14px;font-weight: 400;color: #303133;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, self.widget_space, 250, 0.12);font-weight: 500;}")
        self.rating_sorting_btn.setText(_("Rattings"))
        self.rating_sorting_btn.move(self.type_show_card_widget.width() - self.rating_sorting_btn.width()+self.type_show_card_widget.x()-10,
                                     self.type_name_label.y())
        self.rating_sorting_btn.show()
        self.rating_sorting_btn.clicked.connect(lambda :self.slot_click_sorting_btn(1))
        self.rating_sorting_btn.hide()

        self.download_sorting_btn = QPushButton(self.type_widget)
        self.download_sorting_btn.setFocusPolicy(Qt.NoFocus)
        self.download_sorting_btn.setFixedSize(80,26)
        self.download_sorting_btn.setStyleSheet(
            "QPushButton{font-size: 14px;font-weight: 400;color: #303133;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, self.widget_space, 250, 0.12);font-weight: 500;}")
        self.download_sorting_btn.setText(_("Top app"))
        self.download_sorting_btn.move(self.rating_sorting_btn.x() -self.download_sorting_btn.width(),self.type_name_label.y())
        self.download_sorting_btn.show()
        self.download_sorting_btn.clicked.connect(lambda :self.slot_click_sorting_btn(0))
        self.download_sorting_btn.hide()

        self.popularity_sorting_btn = QPushButton(self.type_widget)
        self.popularity_sorting_btn.setFocusPolicy(Qt.NoFocus)
        self.popularity_sorting_btn.setFixedSize(80,26)
        self.popularity_sorting_btn.setStyleSheet(
            "QPushButton{font-size: 14px;font-weight: 400;color: #303133;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, self.widget_space, 250, 0.12);font-weight: 500;}")
        self.popularity_sorting_btn.setText(_("Default"))
        self.popularity_sorting_btn.move(self.download_sorting_btn.x() -self.download_sorting_btn.width(),self.type_name_label.y())
        self.popularity_sorting_btn.show()
        self.popularity_sorting_btn.clicked.connect(lambda :self.slot_click_sorting_btn(2))
        self.popularity_sorting_btn.hide()


        self.btn_split_line = QLabel(self.type_widget)
        self.btn_split_line.resize(1,12)
        self.btn_split_line.setStyleSheet("QLabel{border: 1px solid #999999;background-color: #999999;}")
        self.btn_split_line.move(self.download_sorting_btn.x()-self.btn_split_line.width()-6,int((self.popularity_sorting_btn.height()-self.btn_split_line.height())/2)+self.popularity_sorting_btn.y())
        self.btn_split_line.hide()

        self.all_check_btn = QPushButton(self.type_widget)
        self.all_check_btn.setFocusPolicy(Qt.NoFocus)
        self.all_check_btn.setText(_("Select all"))
        self.all_check_btn.setFixedSize(80, 26)
        self.all_check_btn.setStyleSheet(
            "QPushButton{font-size: 14px;font-weight: 400;color: #303133;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, self.widget_space, 250, 0.12);font-weight: 500;}")

        self.one_click_download = QPushButton(self.type_widget)
        self.one_click_download.setFocusPolicy(Qt.NoFocus)
        self.one_click_download.setText(_("all down"))
        self.one_click_download.adjustSize()
        self.one_click_download.setFixedSize(80,26)
        self.one_click_download.setStyleSheet("QPushButton{border:0px;font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;border:0px solid #ff0000;}QPushButton:hover{background: rgba(55, self.widget_space, 250, 0.12);border-radius: 12px;}QPushButton:pressed{background: rgba(55, self.widget_space, 250, 0.12);border-radius: 12px;}")
        self.one_click_download.move(self.choose_sorting_type_btn.x() -self.one_click_download.width(),self.type_name_label.y())
        self.all_check_btn.move(self.one_click_download.x() - self.all_check_btn.width(), self.type_name_label.y())
        self.one_click_download.setEnabled(False)

        self.loadingmove = QMovie("./res/loadgif2.gif")
        self.loadingmove_dark = QMovie("./res/loadgifdark.gif")
        self.loadinglabel = QLabel(self)
        self.loadinglabel.setAttribute(Qt.WA_TranslucentBackground)
        self.loadinglabel.resize(2000,2000)
        self.loadinglabel.move(int((self.width()-self.loadinglabel.width())/2),int((self.height()-self.loadinglabel.height())/2))
        self.loadinglabel.setAlignment(Qt.AlignCenter)
        if(Globals.THEME_COLOR == 0):
            self.loadinglabel.setMovie(self.loadingmove)
            self.loadingmove.start()
        else:
            self.loadinglabel.setMovie(self.loadingmove_dark)
            self.loadingmove_dark.start()
        self.loadinglabel.hide()


        self.scrollArea = QScrollArea(self)
        self.scrollArea.setWidget(self.type_widget)
        self.scrollArea.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.scrollArea.horizontalScrollBar().setEnabled(False)
        self.scrollArea.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.scrollArea.setGeometry(0, int(self.btn_widget.height()+self.btn_widget.y()+16), int(self.type_widget.width()-10), int(self.height()-self.btn_widget.height()-10))
        self.type_widget.resize(self.width(),int(self.type_name_label.y()+self.type_name_label.height()+self.type_show_card_widget.height()))
        self.scrollArea.show()
        self.re_move()

    #
    #函数：绘制圆角图片
    #
    def PixmapToRound(self,src,x,y):
        pixmapa = QPixmap(src)
        pixmap = QPixmap(x, y)
        pixmap.fill(Qt.transparent)
        painter = QPainter(pixmap)
        painter.setRenderHint(QPainter.Antialiasing, True)
        # painter.setRenderHints(QPainter.Antialiasing | QPainter.SmoothPixmapTransform)
        path = QPainterPath()
        rect = QRectF(0,0,x,y)
        path.addRoundedRect(rect, 12.0, 12.0)
        painter.setClipPath(path)
        painter.drawPixmap(0, 0, x, y, pixmapa)

        return pixmap

    #
    #函数：初始化界面的样式
    #
    def init_style(self):
        if(Globals.THEME_COLOR == 0):
            self.setStyleSheet("QFrame{border-bottom-left-radius:12px;border-bottom-right-radius:12px; background-color:#FFFFFF;}")
            self.btn_widget.setStyleSheet("QWidget{background-color:#ffffff;border-radius:0px;}")
            # self.type_widget.setStyleSheet("QWidget{background-color:#ffffff;border-radius:0px;}")
            # self.type_show_card_widget.setStyleSheet("QWidget{background-color:#ffffff;border-radius:0px;}")
            self.type_name_label.setStyleSheet("QLabel{font-size: 24px;font-weight: 500;color: #303133;line-height: 42px;}")
            self.type_sorting_btn.setStyleSheet(
                "QPushButton{font-size: 14px;font-weight: 400;color: #3790FA;line-height: 20px;background: transparent;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: transparent;font-weight: 500;}")
            self.pull_down_button.setStyleSheet(
                "QPushButton{border-radius: 4px;border:0px;background-image:url('res/show_search_black.png')}QPushButton:hover{border:0px;background-image:url('res/show_search_clicked.png')}QPushButton:pressed{border:0px;background-image:url('res/show_search_clicked.png')}")
            if(self.one_click_download.isEnabled()):
                self.one_click_download.setStyleSheet("QPushButton{border:0px;font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;border:0px solid #ff0000;}QPushButton:hover{background: rgba(55, 144, 250, 0.12);border-radius: 12px;}QPushButton:pressed{background: rgba(55, 144, 250, 0.12);border-radius: 12px;}")
            else:
                self.one_click_download.setStyleSheet(
                        "QPushButton{font-size:14px;font-weight: 500;color: #8f9399;line-height: 20px;border:0px solid #ff0000;}")
            self.all_check_btn.setStyleSheet(
                "QPushButton{font-size: 14px;border:0px;font-weight: 500;color: #606265;line-height: 20px;}QPushButton:hover{background: rgba(55, 144, 250, 0.12);color: #3790FA;border-radius: 12px;}PushButton:pressed{background: rgba(55, 144, 250, 0.12);color: #3790FA;border-radius: 12px;}")
            self.loadinglabel.setMovie(self.loadingmove)
            self.type_sorting_btn.setStyleSheet(
                "QPushButton{font-size: 14px;font-weight: 400;color: #3790FA;line-height: 20px;background: transparent;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: transparent;font-weight: 500;}")
            self.pull_down_button.setStyleSheet(
                "QPushButton{border-radius: 4px;border:0px;background-image:url('res/show_search_black.png')}QPushButton:hover{border:0px;background-image:url('res/show_search_clicked.png')}QPushButton:pressed{border:0px;background-image:url('res/show_search_clicked.png')}")
            self.loadingmove.start()
            self.scrollArea.verticalScrollBar().setStyleSheet("QScrollBar:vertical{margin:0px 0px 0px 0px;background-color:transparent;border:0px;width:6px;border-radius: 3px;}\
                                QScrollBar::handle:vertical{background-color:#C2C2C2;width:6px;border-radius:3px;} QScrollBar::handle:vertical:hover{background-color:#8F9399;opacity:0.1;width:6px;border-radius:3px;}  QScrollBar::handle:vertical:pressed{background-color:#8F9399;width:6px;border-radius:3px;}\
                                QScrollBar::sub-line:vertical{subcontrol-origin:margin;border:0px solid red;height:13px}\
                                QScrollBar::up-arrow:vertical{subcontrol-origin:margin;background-color:rgba(0,0,0,0);height:13px}\
                                QScrollBar::sub-page:vertical{background-color:transparent;}\
                                QScrollBar::add-page:vertical{background-color:transparent;}\
                                QScrollBar::down-arrow:vertical{background-color:rgba(0,0,0,0);}\
                                QScrollBar::add-line:vertical{subcontrol-origin:margin;border:0px solid green;height:13px}")
            for btn in self.btn_widget.btnGroup.buttons():
                if(self.type_name_label.text() == btn.text_label.text()):
                    btn.setStyleSheet("QPushButton{border-radius: 12px;font-size: 14px;font-weight: 400;line-height: 20px;text-align: center;background-color: #3790FA;color: #ffffff;text-align : center;}")
                else:
                    btn.setStyleSheet("QPushButton{background-color:transparent;border-radius: 12px;font-size: 14px;font-weight: 400;color: #606265;text-align: center;line-height: 20px;text-align : center;}QPushButton:hover{background: rgba(55, 144, 250, 0.12);color: #3790FA;}QPushButton:pressed{background-color: #3790FA;color: #ffffff;}")
                    btn.text_label.setStyleSheet(
                        "QLabel{font-size: 14px;font-weight: 400;color: #606265;line-height: 14px;text-align : center;}")
        else:
            self.setStyleSheet("QFrame{border-bottom-left-radius:12px;border-bottom-right-radius:12px; background-color:#1F2022;}")
            self.btn_widget.setStyleSheet("QWidget{background-color:#1F2022;border-radius:0px;}")
            # self.type_widget.setStyleSheet("QWidget{background-color:#1F2022;border-radius:0px;}")
            # self.type_show_card_widget.setStyleSheet("QWidget{background-color:#1F2022;border-radius:0px;}")
            self.type_name_label.setStyleSheet("QLabel{font-size: 24px;font-weight: 500;color: #F9F9F9;line-height: 42px;}")
            if (self.one_click_download.isEnabled()):
                self.one_click_download.setStyleSheet(
                    "QPushButton{border:0px;font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;border:0px solid #ff0000;}QPushButton:hover{background: rgba(55, 144, 250, 0.12);border-radius: 12px;}QPushButton:pressed{background: rgba(55, 144, 250, 0.12);border-radius: 12px;}")
            else:
                self.one_click_download.setStyleSheet(
                    "QPushButton{font-size:14px;font-weight: 500;color: #8f9399;line-height: 20px;border:0px solid #ff0000;}")
            self.all_check_btn.setStyleSheet("QPushButton{font-size: 14px;border:0px;font-weight: 500;color: #F9F9F9;line-height: 20px;}QPushButton:hover{background: rgba(55, 144, 250, 0.12);color: #3790FA;border-radius: 12px;}PushButton:pressed{background: rgba(55, 144, 250, 0.12);color: #3790FA;border-radius: 12px;}")
            self.loadinglabel.setMovie(self.loadingmove_dark)
            self.loadingmove_dark.start()
            self.scrollArea.verticalScrollBar().setStyleSheet("QScrollBar:vertical{margin:0px 0px 0px 0px;background-color:transparent;border:0px;width:6px;border-radius: 3px;}\
                                QScrollBar::handle:vertical{background-color:#6B6B6B;width:6px;border-radius:3px;} QScrollBar::handle:vertical:hover{background-color:#959595;opacity:0.1;width:6px;border-radius:3px;}  QScrollBar::handle:vertical:pressed{background-color:#959595;width:6px;border-radius:3px;}\
                                QScrollBar::sub-line:vertical{subcontrol-origin:margin;border:0px solid red;height:13px}\
                                QScrollBar::up-arrow:vertical{subcontrol-origin:margin;background-color:rgba(0,0,0,0);height:13px}\
                                QScrollBar::sub-page:vertical{background-color:transparent;}\
                                QScrollBar::add-page:vertical{background-color:transparent;}\
                                QScrollBar::down-arrow:vertical{background-color:rgba(0,0,0,0);}\
                                QScrollBar::add-line:vertical{subcontrol-origin:margin;border:0px solid green;height:13px}")
            for btn in self.btn_widget.btnGroup.buttons():
                if(self.type_name_label.text() == btn.text_label.text()):
                    btn.setStyleSheet("QPushButton{border-radius: 12px;font-size: 14px;font-weight: 400;line-height: 20px;text-align: center;background-color: #3790FA;color: #ffffff;text-align : center;}")
                else:
                    btn.setStyleSheet(
                        "QPushButton{background-color:transparent;border-radius: 12px;font-size: 14px;font-weight: 400;color: #8F9399;text-align: center;line-height: 20px;text-align : center;}QPushButton:hover{background: rgba(55, 144, 250, 0.12);color: #3790FA;}QPushButton:pressed{background-color: #3790FA;color: #ffffff;}")

                    btn.text_label.setStyleSheet(
                            "QLabel{font-size: 14px;font-weight: 400;color: #8F9399;line-height: 14px;text-align : center;}")
        # if (Globals.SORTING_METHOD == 0):
        #     if (Globals.THEME_COLOR == 0):
        #         self.download_sorting_btn.setStyleSheet(
        #             "QPushButton{font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;background: rgba(55, 144, 250, 0.12);border-radius: 13px;}")
        #         self.rating_sorting_btn.setStyleSheet(
        #             "QPushButton{font-size: 14px;font-weight: 400;color: #303133;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, 144, 250, 0.12);font-weight: 500;}")
        #         self.popularity_sorting_btn.setStyleSheet(
        #             "QPushButton{font-size: 14px;font-weight: 400;color: #303133;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, 144, 250, 0.12);font-weight: 500;}")
        #     else:
        #         self.download_sorting_btn.setStyleSheet(
        #             "QPushButton{font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;background: rgba(55, 144, 250, 0.12);border-radius: 13px;}")
        #         self.rating_sorting_btn.setStyleSheet(
        #             "QPushButton{font-size: 14px;font-weight: 400;color: #C0C4CC;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, 144, 250, 0.12);font-weight: 500;}")
        #         self.popularity_sorting_btn.setStyleSheet(
        #             "QPushButton{font-size: 14px;font-weight: 400;color: #C0C4CC;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, 144, 250, 0.12);font-weight: 500;}")
        # elif (Globals.SORTING_METHOD == 1):
        #     if (Globals.THEME_COLOR == 0):
        #         self.rating_sorting_btn.setStyleSheet(
        #             "QPushButton{font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;background: rgba(55, 144, 250, 0.12);border-radius: 13px;}")
        #         self.download_sorting_btn.setStyleSheet(
        #             "QPushButton{font-size: 14px;font-weight: 400;color: #303133;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, 144, 250, 0.12);font-weight: 500;}")
        #         self.popularity_sorting_btn.setStyleSheet(
        #             "QPushButton{font-size: 14px;font-weight: 400;color: #303133;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, 144, 250, 0.12);font-weight: 500;}")
        #     else:
        #         self.rating_sorting_btn.setStyleSheet(
        #             "QPushButton{font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;background: rgba(55, 144, 250, 0.12);border-radius: 13px;}")
        #         self.download_sorting_btn.setStyleSheet(
        #             "QPushButton{font-size: 14px;font-weight: 400;color: #C0C4CC;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, 144, 250, 0.12);font-weight: 500;}")
        #         self.popularity_sorting_btn.setStyleSheet(
        #             "QPushButton{font-size: 14px;font-weight: 400;color: #C0C4CC;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, 144, 250, 0.12);font-weight: 500;}")
        # else:
        #     if (Globals.THEME_COLOR == 0):
        #         self.popularity_sorting_btn.setStyleSheet(
        #             "QPushButton{font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;background: rgba(55, 144, 250, 0.12);border-radius: 13px;}")
        #         self.download_sorting_btn.setStyleSheet(
        #             "QPushButton{font-size: 14px;font-weight: 400;color: #303133;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, 144, 250, 0.12);font-weight: 500;}")
        #         self.rating_sorting_btn.setStyleSheet(
        #             "QPushButton{font-size: 14px;font-weight: 400;color: #303133;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, 144, 250, 0.12);font-weight: 500;}")
        #     else:
        #         self.popularity_sorting_btn.setStyleSheet(
        #             "QPushButton{font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;background: rgba(55, 144, 250, 0.12);border-radius: 13px;}")
        #         self.download_sorting_btn.setStyleSheet(
        #             "QPushButton{font-size: 14px;font-weight: 400;color: #C0C4CC;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, 144, 250, 0.12);font-weight: 500;}")
        #         self.rating_sorting_btn.setStyleSheet(
        #             "QPushButton{font-size: 14px;font-weight: 400;color: #C0C4CC;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, 144, 250, 0.12);font-weight: 500;}")


    #
    #函数：重新设置界面的控件位置和大小
    #
    def re_move(self):
        self.btn_widget.reload_btn()
        self.btn_widget.resize((self.btn_widget.visiblecategorycount - 1) * (
                self.btn_widget.itemwidth + self.btn_widget.categoryspacing) + self.btn_widget.itemwidth, 64)
        self.btn_widget.move(int((self.width() - self.btn_widget.width()) / 2), 20)
        self.type_widget.resize(self.width(), self.height() - self.btn_widget.height())
        self.category_ad_card.resize(self.type_widget.width()-self.widget_space-self.widget_space, round(self.category_ad_card_height * self.m_parent.proportion))
        self.category_ad_card.re_move(self.m_parent.proportion)
        self.preview_ad_card.resize(self.type_widget.width() - self.widget_space * 2, self.category_ad_card.height())
        photo = "./data/ads/preview_banner.png"
        pix2 = QPixmap(photo)
        fitpixmap_userIcon = pix2.scaled(self.preview_ad_card.width(), self.preview_ad_card.height(),
                                         Qt.IgnoreAspectRatio, Qt.SmoothTransformation)
        fitpixmap_userIcon = self.PixmapToRound(fitpixmap_userIcon, self.preview_ad_card.width(),
                                                self.preview_ad_card.height())
        self.preview_ad_card.setPixmap(fitpixmap_userIcon)
        self.type_name_label.move(self.widget_space, self.category_ad_card.height() + 30)
        self.type_widget.resize(self.type_widget.width(),
                                int(self.type_name_label.y() + self.type_name_label.height() + self.type_show_card_widget.height()))
        self.type_show_card_widget.resize(self.type_widget.width() - self.widget_space - self.widget_space,
                                          self.type_widget.height() - self.type_name_label.height())

        self.choose_sorting_type_btn.move(int(self.type_show_card_widget.width() - self.choose_sorting_type_btn.width()+self.type_show_card_widget.x()-10),int(self.type_name_label.y()))
        self.rating_sorting_btn.move(int(self.type_show_card_widget.width() - self.rating_sorting_btn.width() + self.type_show_card_widget.x() - 10),int(self.type_name_label.y()))
        self.download_sorting_btn.move(int(self.rating_sorting_btn.x() - self.download_sorting_btn.width()),
                                       int(self.type_name_label.y()))
        self.btn_split_line.move(int(self.popularity_sorting_btn.x() - self.btn_split_line.width() - 6),
                                 int((self.popularity_sorting_btn.height() - self.btn_split_line.height()) / 2) +self.popularity_sorting_btn.y())
        self.one_click_download.move(int(self.choose_sorting_type_btn.x() - self.one_click_download.width()),int(self.type_name_label.y()))
        self.all_check_btn.move(int(self.one_click_download.x() - self.all_check_btn.width()), int(self.type_name_label.y()))
        self.loadinglabel.move(int((self.width() - self.loadinglabel.width()) / 2),int((self.height() - self.loadinglabel.height()) / 2))

        self.scrollArea.setGeometry(0, int(self.btn_widget.height()+self.btn_widget.y()+16), int(self.type_widget.width()-10), int(self.height()-self.btn_widget.height()-self.btn_widget.y()-30))
        self.type_show_card_widget.move(self.widget_space, int(self.type_name_label.y() + self.type_name_label.height()))
        self.reload_card(self.type_show_card_widget,self.scrollArea)
        self.type_widget.resize(self.type_widget.width(),
                                int(self.type_name_label.y() + self.type_name_label.height() + self.type_show_card_widget.height()))

    #
    #函数：重新设置卡片
    #
    def reload_card(self,widget,scrollArea):
        cards = widget.children()
        if (len(cards) != 0):
            h_num = int(widget.width()/(self.card_width+self.card_space))
            get_card_width = int((widget.width() - (self.card_space * (h_num - 1)) - 20) / h_num)
            proportion = get_card_width / widget.width()
            get_card_height = self.card_height
            v_num = 0
            num = 0
            for i in range(len(cards)):
                card = cards[i]
                card.resize(get_card_width,get_card_height)
                card.re_move()
                if (num < h_num):
                    if (num == 0):
                        card.move(0, 20 + (20 + card.height()) * v_num)
                        widget.resize(widget.width(), 20 + (20 + card.height()) * (v_num + 1))
                    else:
                        card.move(num * (card.width() + self.card_space) + 5, 20 + (20 + card.height()) * v_num)
                    num = num + 1
                else:
                    num = 0
                    v_num = v_num + 1
                    if (num == 0):
                        card.move(0, 20+ (20 + card.height()) * v_num)
                        widget.resize(widget.width(),  (20 + card.height()) * (v_num + 1))
                    else:
                        card.move(num * (card.width() + self.card_space), 20 + (20 + card.height()) * v_num)
                    num = num + 1

    #
    #函数：处理点击分类界面的类型按钮
    #
    def slot_clicked_btn(self,btn_name):
        self.click_type_btn.emit(btn_name,True)


    def slot_click_sorting_btn(self,sort_type,need_emit_signal = True):
        #sort_type:0(下载排行)，1（评分排行）2（热度排行）
        # if(Globals.SORTING_METHOD == 0):
        #     Globals.SORTING_METHOD = 1
        # else:
        #     Globals.SORTING_METHOD = 0
        if(sort_type == 0):
            Globals.SORTING_METHOD = 0
            if(Globals.THEME_COLOR == 0):
                self.download_sorting_btn.setStyleSheet("QPushButton{font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;background: rgba(55, 144, 250, 0.12);border-radius: 13px;}")
                self.rating_sorting_btn.setStyleSheet("QPushButton{font-size: 14px;font-weight: 400;color: #303133;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, 144, 250, 0.12);font-weight: 500;}")
                self.popularity_sorting_btn.setStyleSheet(
                    "QPushButton{font-size: 14px;font-weight: 400;color: #303133;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, 144, 250, 0.12);font-weight: 500;}")
            else:
                self.download_sorting_btn.setStyleSheet(
                    "QPushButton{font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;background: rgba(55, 144, 250, 0.12);border-radius: 13px;}")
                self.rating_sorting_btn.setStyleSheet(
                    "QPushButton{font-size: 14px;font-weight: 400;color: #C0C4CC;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, 144, 250, 0.12);font-weight: 500;}")
                self.popularity_sorting_btn.setStyleSheet(
                    "QPushButton{font-size: 14px;font-weight: 400;color: #C0C4CC;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, 144, 250, 0.12);font-weight: 500;}")
        elif(sort_type == 1):
            Globals.SORTING_METHOD = 1
            if(Globals.THEME_COLOR == 0):
                self.rating_sorting_btn.setStyleSheet(
                    "QPushButton{font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;background: rgba(55, 144, 250, 0.12);border-radius: 13px;}")
                self.download_sorting_btn.setStyleSheet(
                    "QPushButton{font-size: 14px;font-weight: 400;color: #303133;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, 144, 250, 0.12);font-weight: 500;}")
                self.popularity_sorting_btn.setStyleSheet(
                    "QPushButton{font-size: 14px;font-weight: 400;color: #303133;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, 144, 250, 0.12);font-weight: 500;}")
            else:
                self.rating_sorting_btn.setStyleSheet(
                    "QPushButton{font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;background: rgba(55, 144, 250, 0.12);border-radius: 13px;}")
                self.download_sorting_btn.setStyleSheet(
                    "QPushButton{font-size: 14px;font-weight: 400;color: #C0C4CC;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, 144, 250, 0.12);font-weight: 500;}")
                self.popularity_sorting_btn.setStyleSheet(
                    "QPushButton{font-size: 14px;font-weight: 400;color: #C0C4CC;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, 144, 250, 0.12);font-weight: 500;}")
        else:
            Globals.SORTING_METHOD = 2
            if (Globals.THEME_COLOR == 0):
                self.popularity_sorting_btn.setStyleSheet(
                    "QPushButton{font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;background: rgba(55, 144, 250, 0.12);border-radius: 13px;}")
                self.download_sorting_btn.setStyleSheet(
                    "QPushButton{font-size: 14px;font-weight: 400;color: #303133;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, 144, 250, 0.12);font-weight: 500;}")
                self.rating_sorting_btn.setStyleSheet(
                    "QPushButton{font-size: 14px;font-weight: 400;color: #303133;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, 144, 250, 0.12);font-weight: 500;}")
            else:
                self.popularity_sorting_btn.setStyleSheet(
                    "QPushButton{font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;background: rgba(55, 144, 250, 0.12);border-radius: 13px;}")
                self.download_sorting_btn.setStyleSheet(
                    "QPushButton{font-size: 14px;font-weight: 400;color: #C0C4CC;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, 144, 250, 0.12);font-weight: 500;}")
                self.rating_sorting_btn.setStyleSheet(
                    "QPushButton{font-size: 14px;font-weight: 400;color: #C0C4CC;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, 144, 250, 0.12);font-weight: 500;}")
        # self.check_choose_btn_type()
        if(need_emit_signal == True):
                self.click_type_btn.emit(self.type_name_label.text(),False)

    #
    #函数：处理点击选择排行类型
    #
    def slot_choose_sorting_type(self):
        if(not hasattr(self,"select_widget")):
            item_name_list = [_("Default"),_("Top app"),_("Rattings")]
            self.select_widget = Select_Widget(102,30,3,item_name_list,False,self.type_widget)
            self.select_widget.click_item.connect(self.slot_click_select_list_item)
            item_count = self.select_widget.item_list.count()
            for i in range(item_count):
                item = self.select_widget.item_list.item(i)
                get_item_widget = self.select_widget.item_list.itemWidget(item)
                if(Globals.SORTING_METHOD == 2):
                    if(get_item_widget.item_name_label.text() == _("Default")):
                        get_item_widget.icon_label.show()
                    else:
                        get_item_widget.icon_label.hide()
                elif(Globals.SORTING_METHOD == 1):
                    if (get_item_widget.item_name_label.text() == _("Top app")):
                        get_item_widget.icon_label.show()
                    else:
                        get_item_widget.icon_label.hide()
                else:
                    if (get_item_widget.item_name_label.text() == _("Rattings")):
                        get_item_widget.icon_label.show()
                    else:
                        get_item_widget.icon_label.hide()
        self.select_widget.move(self.x()+self.type_widget.x()+self.choose_sorting_type_btn.x()+self.choose_sorting_type_btn.width()-self.select_widget.width()+6,self.y()+self.type_widget.y()+self.choose_sorting_type_btn.y()+self.choose_sorting_type_btn.height()+self.scrollArea.verticalScrollBar().value()-6)
        self.select_widget.slot_show_dark_mode()
        self.select_widget.show()

    #
    #函数：处理按钮的显示以及select_widget勾选状态
    #
    def check_choose_btn_type(self):
        if(Globals.SORTING_METHOD == 2):
            self.type_sorting_btn.setText(_("Default"))
            self.type_sorting_btn.adjustSize()
        elif(Globals.SORTING_METHOD == 0):
            self.type_sorting_btn.setText(_("Top app"))
            self.type_sorting_btn.adjustSize()
        else:
            self.type_sorting_btn.setText(_("Rattings"))
            self.type_sorting_btn.adjustSize()
        if (hasattr(self, "select_widget")):
            item_count = self.select_widget.item_list.count()
            for i in range(item_count):
                item = self.select_widget.item_list.item(i)
                get_item_widget = self.select_widget.item_list.itemWidget(item)
                if (Globals.SORTING_METHOD == 2):
                    if (get_item_widget.item_name_label.text() == _("Default")):
                        get_item_widget.icon_label.show()
                    else:
                        get_item_widget.icon_label.hide()
                elif (Globals.SORTING_METHOD == 1):
                    if (get_item_widget.item_name_label.text() == _("Top app")):
                        get_item_widget.icon_label.show()
                    else:
                        get_item_widget.icon_label.hide()
                else:
                    if (get_item_widget.item_name_label.text() == _("Rattings")):
                        get_item_widget.icon_label.show()
                    else:
                        get_item_widget.icon_label.hide()

    #
    #函数：处理点击list_item
    #
    def slot_click_select_list_item(self,item):
        get_item_widget = self.select_widget.item_list.itemWidget(item)
        text = get_item_widget.item_name_label.text()
        get_item_widget.icon_label.show()
        self.type_sorting_btn.setText(text)
        self.type_sorting_btn.adjustSize()
        self.select_widget.hide()
        if(text == _("Default")):
            self.slot_click_sorting_btn(2)
        elif(text == _("Top app")):
            self.slot_click_sorting_btn(0)
        else:
            self.slot_click_sorting_btn(1)


    #
    #函数：处理点击广告
    #
    def slot_click_ad_card(self,appmsg,need_send_exposure_rate):
        if(type(appmsg) == Application):
            self.click_category_ad.emit(appmsg,need_send_exposure_rate)
        else:
            self.click_category_ad_apk.emit(appmsg,need_send_exposure_rate)

    #
    #函数：处理点击源中没有的广告
    #
    def slot_click_none_card(self):
        self.click_none_card.emit()

    def mouseReleaseEvent(self, event):
        self.m_parent.mw.hide_tip_widget()
        if (hasattr(self, "select_widget")):
            self.select_widget.hide()

    # 鼠标进入事件

    def enterEvent(self, event):
        self.status = BUTTON_ENTER
        self.scrollArea.verticalScrollBar().show()
        # self.scrollArea.show()

    #
    # 函数：鼠标离开事件
    #

    def leaveEvent(self, event):
        self.status = BUTTON_LEAVE
        self.scrollArea.verticalScrollBar().hide()

    #
    # 函数：鼠标按下事件
    #
    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            self.status = BUTTON_PRESSED
        else:
            event.ignore()

    #
    # 函数名:设置滚动条起始位置
    # Function:set verticalSscrollBar starting point
    #
    def scrollToTop(self):
        vsb = self.scrollArea.verticalScrollBar()
        vsb.setValue(0)

class CategoryBar(QWidget):

    categorycount = 0
    itemwidth = 52
    categoryspacing = 8
    first_btn = ""
    click_categoy = pyqtSignal(str,bool)

    def __init__(self,category_list,parent=None,is_kydroid = False):
        QWidget.__init__(self, parent)

        # self.caslot_btn_clickedtegorytab = QLabel(self)`
        self.visiblecategorycount = 0
        self.btn_x = 0
        self.btn_y = 0
        self.is_kydroid = is_kydroid
        self.m_parent = parent
        self.category_list = category_list
        self.categoryPanel = QWidget(self)
        self.btnGroup = QButtonGroup(self.categoryPanel)
        self.btnGroup.buttonClicked.connect(self.slot_btn_clicked)
        self.init_categories(self.category_list)

    #
    # 函数名:进入控件
    # Function:enter control
    #
    def enterEvent(self, event):
        pass

    #
    # 函数名:离开控件
    # Function:leave control
    #
    def leaveEvent(self, event):
        pass

    #
    # 函数名:初始化分类
    # Function:init categories
    #
    def init_categories(self,category_list):
        cat_list = category_list
        #安卓界面显示全部分类
        if(self.is_kydroid == True):
            if (self.visiblecategorycount == 0):
                item = {'is_visible': 1, 'id': 10000, 'name': 'all', 'displayname': '全部', 'priority': 5}
                self.add_category(item)
        for item in cat_list:
            category = item["name"]
            if(item["is_visible"] == 1):
                if(item["id"] == 1):
                    if(self.is_kydroid == False):
                        self.visiblecategorycount = self.visiblecategorycount + 1
                        self.add_category(item)
                    else:
                        continue
                else:
                    if(self.is_kydroid):
                        for i in Globals.KYDROID_CATEGORY:
                            if(int(item["id"]) == int(i)):
                                self.visiblecategorycount = self.visiblecategorycount + 1
                                self.add_category(item)
                                break
                    else:
                        self.visiblecategorycount = self.visiblecategorycount + 1
                        self.add_category(item)
        self.setFixedWidth(self.btn_x)

    #
    # 函数名:添加分类
    # Function:add category
    #
    def add_category(self, category):
        btnCategory = CategoryButton(category["name"], category["displayname"], self.categoryPanel)
        # btnCategory.enter_btn.connect(self.slot_btn_clicked)
        # btnCategory.clicked.connect(lambda: self.slot_btn_clicked(btnCategory))
        btnCategory.click_btn.connect(self.slot_btn_clicked)
        self.btnGroup.addButton(btnCategory)
        x = self.btn_x
        if(self.m_parent.width() -self.m_parent.widget_space <self.btn_x + btnCategory.width()):
            btnCategory.move(0, self.btn_y + 20+btnCategory.height())
            self.setFixedWidth(self.btn_x)
            self.btn_x = 0
            self.btn_y = self.btn_y + btnCategory.height() +20
            self.setFixedHeight(self.btn_y + btnCategory.height())
        else:
            btnCategory.move(self.btn_x, self.btn_y)
        self.btn_x = self.btn_x + btnCategory.width() + self.categoryspacing
        self.categorycount = self.categorycount + 1
        if(self.categorycount == 1):
            self.first_btn = btnCategory.text_label.text()
            btnCategory.is_clicked = True
            btnCategory.setStyleSheet("QPushButton{border-radius: 12px;font-size: 14px;font-weight: 400;line-height: 20px;text-align: center;background-color: #3790FA;color: #ffffff;text-align : center;}")
            btnCategory.text_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #ffffff;line-height: 14px;text-align : center;}")
            zh_local = os.getenv("LANG")
            if ("zh_CN" in zh_local):
                self.click_categoy.emit(category["displayname"],True)
            else:
                self.click_categoy.emit(category["name"], True)
                # btnCategory.setText(category["name"])

        if self.visiblecategorycount == self.categorycount:
            return

    #
    # 函数名:控件左侧
    # Function:control left
    #
    def scrollToLeft(self):
        pass

    #
    # 函数名:控件右侧
    # Function:control right
    #
    def scrollToRight(self):
        pass


    #
    # 函数名:分类数量
    # Function:count the categories
    #
    def count(self):
        return self.categorycount

    #
    # 函数名:删除item
    # Function:remove item
    #
    def clear(self):
        categories = self.categoryPanel.children()
        for category in categories:
            sip.delete(category)

    #
    # 函数名:点击控件
    # Function:click control
    #
    def slot_btn_clicked(self, btn,all_reflash = True):
        btns = self.btnGroup.buttons()
        for b in btns:
            if(b != btn):
                b.hide()
                b.is_clicked = False
                if(Globals.THEME_COLOR == 0):
                    b.setStyleSheet("QPushButton{background-color:transparent;border-radius: 12px;font-size: 14px;font-weight: 400;color: #606265;text-align: center;line-height: 20px;text-align : center;}QPushButton:hover{background: rgba(55, 144, 250, 0.12);color: #3790FA;}QPushButton:pressed{background-color: #3790FA;color: #ffffff;}")
                    b.text_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #606265;line-height: 14px;text-align : center;}")
                else:
                    b.setStyleSheet(
                        "QPushButton{background-color:transparent;border-radius: 12px;font-size: 14px;font-weight: 400;color: #8F9399;text-align: center;line-height: 20px;text-align : center;}QPushButton:hover{background: rgba(55, 144, 250, 0.12);color: #3790FA;}QPushButton:pressed{background-color: #3790FA;color: #ffffff;}")
                    b.text_label.setStyleSheet(
                        "QLabel{font-size: 14px;font-weight: 400;color: #8F9399;line-height: 14px;text-align : center;}")
                b.show()
        btn.hide()
        btn.setStyleSheet("QPushButton{border-radius: 12px;font-size: 14px;font-weight: 400;line-height: 20px;background-color: #3790FA;text-align: center;color: #ffffff;text-align : center;}")
        btn.text_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #FFFFFF;line-height: 14px;text-align : center;}")
        btn.show()
        self.m_parent.type_name_label.setText(btn.text_label.text())
        self.m_parent.type_name_label.adjustSize()
        category = str(btn.text_label.text())

        # self.categorytab.move(btn.x(), self.categorytab.y())
        # self.categorytab.show()
        if (self.is_kydroid == False):
            # if(Globals.NOW_CATEGORY != btn.text_label.text()):
            self.click_categoy.emit(category,all_reflash)
        else:
            # if (Globals.NOW_KYDROID_CATEGORY != btn.text_label.text()):
            self.click_categoy.emit(category, all_reflash)
            Globals.NOW_KYDROID_CATEGORY = btn.text_label.text()

    #
    #函数：界面修改大小，修改分类按钮的位置
    #
    def reload_btn(self):
        btns = self.btnGroup.buttons()
        x = 0
        y = 0
        for b in btns:
            if (self.m_parent.width() - self.m_parent.widget_space < x + b.width()):
                b.move(0, y + 20+b.height())
                self.setFixedWidth(x)
                x = 0
                y = y + b.height() + 20
                self.setFixedHeight(y + b.height())
            else:
                b.move(x, y)
            x = x + b.width() + self.categoryspacing
            if(y == 0):
                self.setFixedSize(x ,b.height())
        self.categoryPanel.resize(self.width()-self.categoryspacing,self.height())
        # print(self.height())

    # add by kobe
    #
    # 函数名:重置类别栏
    # Function:reset categorybar
    #
    def reset_categorybar(self):
        # self.categorytab.hide()
        # self.categorytab.setStyleSheet("QLabel{background-image:url('res/categorytab.png');background-position:center;}")
        btns = self.btnGroup.buttons()
        for b in btns:
            if(Globals.THEME_COLOR == 0):
                b.setStyleSheet("QPushButton{border-radius: 12px;background-color:transparent;font-size: 14px;font-weight: 400;color: #606265;text-align: center;line-height: 20px;}QPushButton:hover{background: rgba(55, 144, 250, 0.12);color: #3790FA;}QPushButton:pressed{background-color: #3790FA;color: #ffffff;}")
            else:
                b.setStyleSheet("QPushButton{background-color:transparent;border-radius: 12px;font-size: 14px;font-weight: 400;color: #8F9399;text-align: center;line-height: 20px;text-align : center;}QPushButton:hover{background: rgba(55, 144, 250, 0.12);color: #3790FA;}QPushButton:pressed{background-color: #3790FA;color: #ffffff;}")


class CategoryButton(QPushButton):

    category_name = ''
    display_name = ''
    enter_btn = pyqtSignal(QPushButton)
    click_btn = pyqtSignal(QPushButton)

    def __init__(self, categoryname, displayname, parent=None):
        QPushButton.__init__(self, parent)
        self.is_clicked = False

        self.setFixedSize(52, 24)
        self.setCheckable(True)
        self.setFocusPolicy(Qt.NoFocus)

        self.text_label = QLabel(self)
        self.text_label.setAttribute(Qt.WA_TranslucentBackground)
        self.text_label.setAlignment(Qt.AlignCenter)
        self.text_label.move(12,6)

        self.category_name = categoryname
        self.display_name = displayname
        # if displayname=="特色":
        #     self.setText(_("Feature"))
        # elif displayname=="办公":
        #     self.setText(_("Office"))
        # elif displayname=="开发":
        #     self.setText(_("Develp"))
        # elif displayname=="图像":
        #     self.setText(_("Image"))
        # elif displayname=="影音":
        #     self.setText(_("Audio"))
        # elif displayname=="网络":
        #     self.setText(_("Net"))
        # elif displayname=="游戏":
        #     self.setText(_("Game"))
        # elif displayname=="教育":
        #     self.setText(_("Edu"))
        # elif displayname == "社交":
        #     self.setText(_("Social"))
        # elif displayname == "系统":
        #     self.setText(_("System"))
        # elif displayname == "其它":
        #     self.setText(_("Other"))
        zh_local = os.getenv("LANG")
        if("zh_CN" in zh_local):
            self.text_label.setText(displayname)
        else:
            self.text_label.setText(categoryname)
        if(Globals.THEME_COLOR == 0):
            self.setStyleSheet("QPushButton{background-color:transparent;border-radius: 12px;font-size: 14px;font-weight: 400;color: #606265;line-height: 20px;text-align : center;}QPushButton:hover{background: rgba(55, 144, 250, 0.12);color: #3790FA;}QPushButton:pressed{background-color: #3790FA;color: #ffffff;}")
            self.text_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #606265;line-height: 14px;text-align : center;}")
        else:
            self.setStyleSheet("QPushButton{background-color:transparent;border-radius: 12px;font-size: 14px;font-weight: 400;color: #8F9399;text-align: center;line-height: 20px;text-align : center;}QPushButton:hover{background: rgba(55, 144, 250, 0.12);color: #3790FA;}QPushButton:pressed{background-color: #3790FA;color: #ffffff;}")
            self.text_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #8F9399;line-height: 14px;text-align : center;}")
        self.text_label.adjustSize()
        self.text_label.move(12,int((self.height()-self.text_label.height())/2))
        self.setFixedWidth(self.text_label.width()+24)

    #
    # 函数：鼠标进入事件
    #
    def enterEvent(self, event):
        #self.enter_btn.emit(self)
        if(self.is_clicked == False):
            self.text_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #3790FA;line-height: 14px;text-align : center;}")
        else:
            self.text_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #FFFFFF;line-height: 14px;text-align : center;}")
        self.update()

    def mousePressEvent(self,event):
        if event.button() == Qt.LeftButton:
            self.text_label.setStyleSheet(
                "QLabel{font-size: 14px;font-weight: 400;color: #ffffff;line-height: 14px;text-align : center;}")
            self.setStyleSheet("QPushButton{border-radius: 12px;font-size: 14px;font-weight: 400;line-height: 20px;background-color: #3790FA;text-align: center;color: #ffffff;text-align : center;}")
            self.is_clicked = True
            self.click_btn.emit(self)
        else:
            event.ignore()

    def leaveEvent(self, event):
        if(self.is_clicked == False):
            if (Globals.THEME_COLOR == 0):
                self.text_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #606265;line-height: 14px;text-align : center;}")
            else:
                self.text_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #8F9399;line-height: 14px;text-align : center;}")
        else:
            self.text_label.setStyleSheet(
                "QLabel{font-size: 14px;font-weight: 400;color: #FFFFFF;line-height: 14px;text-align : center;}")
            self.setStyleSheet("QPushButton{border-radius: 12px;font-size: 14px;font-weight: 400;line-height: 20px;background-color: #3790FA;text-align: center;color: #ffffff;text-align : center;}")



#!/usr/bin/python3
# -*- coding: utf-8 -*-

### BEGIN LICENSE

# Copyright (C) 2013 National University of Defense Technology(NUDT) & Kylin Ltd

# Author:
#     Shine Huang<shenghuang@ubuntukylin.com>
# Maintainer:
#     Shine Huang<shenghuang@ubuntukylin.com>

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import platform
import locale
from models.globals import Globals
from utils.log_manager import LogManager
import gettext
gettext.textdomain("kylin-software-center")
_ = gettext.gettext

# new version no longer send dbus id
def get_machine_id():
    # fpath = '/var/lib/dbus/machine-id'
    # if(os.path.exists(fpath) and os.path.isfile(fpath)):
    #     f = open(fpath, 'r')
    #     id = f.read()
    #     f.close()
    #     id = id.replace('\n','')
    #     if(id == ''):
    #         return 'unknown'
    #     else:
    #         return id
    # else:
    #     return 'unknown'

    return 'empty'

# the linux distribution of this machine
def get_distro_info():
    ufpath = '/etc/ubuntukylin-release'
    if(os.path.exists(ufpath) and os.path.isfile(ufpath)):
        uf = open(ufpath)
        lines = uf.readlines()
        rtn = []
        for line in lines:
            kv = line.split('=')
            if (kv[0] == 'DISTRIB_ID'):
                v = kv[1]
                rtn.append(v[:-1])
            if (kv[0] == 'DISTRIB_RELEASE'):
                v = kv[1]
                rtn.append(v[:-1])
            if (kv[0] == 'DISTRIB_CODENAME'):
                v = kv[1]
                rtn.append(v[:-1])
        uf.close()
        return rtn
    else:
        dist = platform.dist()
        distname = dist[0]
        distversion = dist[1]
        distroseries = dist[2]
        return [distname, distversion, distroseries]

# uksc version
def get_uksc_version():
    return Globals.UKSC_VERSION

# get this os language
def get_language():
    return locale.getdefaultlocale()[0]

def get_cpuArchitecture():
    arch = os.popen("dpkg --print-architecture").readline()
    # arch = os.popen("dpkg-architecture |grep DEB_BUILD_ARCH_CPU|awk -F '=' '{print $2}'").readline()
    return arch.strip("\n")

def get_osversion():
    os_version = os.popen("cat /etc/lsb-release|grep DISTRIB_RELEASE|awk -F '=' '{print $2}'").readline()
    return os_version.strip("\n")

def get_appversion():
    appversion = ''
    appversion = os.popen("dpkg -l|grep kylin-software-center|awk -F ' ' '{print $3}'").readline()
    return appversion.strip("\n")


